#/usr/bin/env python
#
# This module contains all the RPC-related functions the RHN code uses
#
# Copyright (c) 2002 Red Hat, Inc.
#
# Author: Mihai Ibanescu <misa@redhat.com>

# $Id: rpclib.py,v 1.25 2002/08/29 22:16:16 misa Exp $

__version__ = "$Revision: 1.25 $"

import string
import transports

# We may have an internalized version of xmlrpclib, we determine that in
# transports
xmlrpclib = transports.xmlrpclib

# Wrappers around xmlrpclib objects
Fault = xmlrpclib.Fault

# XXX Do we want to do it this way, or are we going to use __init__ for this?
ResponseError = xmlrpclib.ResponseError
ProtocolError = xmlrpclib.ProtocolError

getparser = xmlrpclib.getparser

# This is a cut-and-paste of xmlrpclib.ServerProxy, with the data members made
# protected instead of private
# It also adds support for changing the way the request is made (XMLRPC or
# GET)
class Server:
    """uri [,options] -> a logical connection to an XML-RPC server

    uri is the connection point on the server, given as
    scheme://host/target.

    The standard implementation always supports the "http" scheme.  If
    SSL socket support is available (Python 2.0), it also supports
    "https".

    If the target part and the slash preceding it are both omitted,
    "/RPC2" is assumed.

    The following options can be given as keyword arguments:

        transport: a transport factory
        encoding: the request encoding (default is UTF-8)
        verbose: verbosity level
        proxy: use an HTTP proxy
        username: username for authenticated HTTP proxy
        password: password for authenticated HTTP proxy

    All 8-bit strings passed to the server proxy are assumed to use
    the given encoding.
    """

    def __init__(self, uri, transport=None, encoding=None, verbose=0, 
        proxy=None, username=None, password=None, refreshCallback=None,
        progressCallback=None):
        # establish a "logical" server connection

        # get the url
        import urllib
        type, uri = urllib.splittype(uri)
        type = string.lower(type)
        if type not in ("http", "https"):
            raise IOError, "unsupported XML-RPC protocol"
        self._host, self._handler = urllib.splithost(uri)
        if not self._handler:
            self._handler = "/RPC2"

        if transport is None:
            if proxy:
                if type == 'https':
                    transport = transports.SafeProxyTransport(proxy, 
                        proxyUsername=username, proxyPassword=password)
                else:
                    transport = transports.ProxyTransport(proxy, 
                        proxyUsername=username, proxyPassword=password)
            else:
                if type == 'https':
                    transport = transports.SafeTransport()
                else:
                    transport = transports.Transport()

        self._transport = transport

        self._encoding = encoding
        self._verbose = verbose

        self.set_refresh_callback(refreshCallback)
        self.set_progress_callback(progressCallback)

    def set_refresh_callback(self, refreshCallback):
        self._transport.set_refresh_callback(refreshCallback)

    def set_buffer_size(self, bufferSize):
        self._transport.set_buffer_size(bufferSize)

    def set_progress_callback(self, progressCallback, bufferSize=16384):
        self._transport.set_progress_callback(progressCallback, bufferSize)

    def _req_body(self, params, methodname):
        return xmlrpclib.dumps(params, methodname, encoding=self._encoding)
    
    def _request(self, methodname, params):
        # call a method on the remote server

        # Clear the transport headers first
        self._transport.clear_headers()
        
        self._transport.add_header("X-Info",
            'RPC Processor (C) Red Hat, Inc (version %s)' % 
            string.split(__version__)[1])
        # identify the capability set of this client to the server
        self._transport.set_header("X-Client-Version", 1)

        request = self._req_body(params, methodname)

        response = self._transport.request(
            self._host,
            self._handler,
            request,
            verbose=self._verbose
            )

        if len(response) == 1:
            response = response[0]

        return response

    def __repr__(self):
        return (
            "<%s for %s%s>" %
            (self.__class__.__name__, self._host, self._handler)
            )

    __str__ = __repr__

    def __getattr__(self, name):
        # magic method dispatcher
        return xmlrpclib._Method(self._request, name)

    # note: to call a remote object with an non-standard name, use
    # result getattr(server, "strange-python-name")(args)

    def set_transport_flags(self, transfer = 0, encoding = 0):
        if self._transport:
            self._transport.set_transport_flags(transfer, encoding)

    # Allow user-defined additional headers.
    def add_header(self, name, arg):
	if self._transport:
	    self._transport.add_header(name, arg)

    # Sets the i18n options
    def setlang(self, lang):
        if self._transport and hasattr(self._transport, "setlang"):
            self._transport.setlang(lang)
        
    # Sets the CA chain to be used
    def use_CA_chain(self, ca_chain = None):
        raise NotImplementedError, "This method is deprecated"

    def add_trusted_cert(self, certfile):
        if self._transport and hasattr(self._transport, "add_trusted_cert"):
            self._transport.add_trusted_cert(certfile)
        
    def close(self):
        if self._transport:
            self._transport.close()
            self._transport = None

# RHN GET server
class GETServer(Server):
    def __init__(self, uri, transport=None, proxy=None, username=None,
            password=None, client_version=2, headers={}, refreshCallback=None,
            progressCallback=None):
        Server.__init__(self, uri, 
            proxy=proxy,
            username=username,
            password=password,
            transport=transport,
            refreshCallback=refreshCallback,
            progressCallback=progressCallback)
        self._client_version = client_version
        self.__headers = headers
        # Back up the original handler, since we mangle it
        self._orig_handler = self._handler
        # Set the transport method to GET
        self._transport.set_method("GET")

    def _req_body(self, params, methodname):
        if not params or len(params) < 1:
            raise Exception("Required parameter channel not found")
        # Strip the multiple / from the handler
        h_comps = filter(lambda x: x != '', string.split(self._orig_handler, '/'))
        # Set the handler we are going to request
        hndl = h_comps + ["$RHN", params[0], methodname] + list(params[1:])
        self._handler = '/' + string.join(hndl, '/')
        # Add headers
        for h, v in self.__headers.items():
            self._transport.add_header(h, v)
        return ''


def getHeaderValues(headers, name):
    import mimetools
    if not isinstance(headers, mimetools.Message):
        if headers.has_key(name):
            return [headers[name]]
        return []

    return map(lambda x: string.strip(string.split(x, ':', 1)[1]), 
            headers.getallmatchingheaders(name))


def reportError(headers):
    # Reports the error from the headers
    errcode = 0
    errmsg = ""
    s = "X-RHN-Fault-Code"
    if headers.has_key(s):
        errcode = int(headers[s])
    s = "X-RHN-Fault-String"
    if headers.has_key(s):
        _sList = getHeaderValues(headers, s)
        if _sList:
            _s = string.join(_sList, '')
            import base64
            errmsg = "%s" % base64.decodestring(_s)

    return errcode, errmsg

